/************************************
 * File: Direction.h
 *
 * Contains the definition and prototypes for the directionT type.  DO NOT MODIFY
 * THIS FILE.
 */
#ifndef Direction_included
#define Direction_included

/* Type: directionT
 * Values: North, South, East, West, InvalidDirection
 */
enum directionT
{
	North, East, South, West, InvalidDirection
};

/* Type: diagonalDirectionT
 * Values: UpRight (/), DownRight (\)
 */
enum diagonalDirectionT
{
	UpRight, DownRight, InvalidDiagonalDirection
};

/**
 * Returns the opposite direction, throwing a runtime_error if there is no opposite direction.
 */
directionT OppositeDirection(directionT input);

/**
 * Returns the direction 90 degrees counterclockwise from the original direction,
 * throwing a runtime_error if there is no opposite direction.
 */
directionT RotateCounterclockwise(directionT input);

/**
 * Returns the direction 90 degrees clockwise from the original direction,
 * throwing a runtime_error if there is no opposite direction.
 */
directionT RotateClockwise(directionT input);
 

/**
 * Given an x/y pair, updates the pair by moving them in the specified direction.  If the direction
 * is invalid, throws a runtime_error.
 */
void MovePointInDirection(int& x, int& y, directionT direction);

/**
 * Given a direction representing an incoming direction to a mirror running in the specified
 * direction, returns the direction that the beam will be going out.  Thus, if we have an
 * UpRight mirror ( / ) being hit from a beam moving north, the function will return East.
 * Throws an invalid_argument exception if either direction is invalid.
 */
directionT ReflectDirectionOffMirror(directionT incoming, diagonalDirectionT mirrorDirection);

/**
 * Given a character from a world file, returns the corresponding directionT for
 * that character.  Throws an invalid_argument error if the parameter does not
 * correspond to a direction.
 */
directionT CharToDirection(char ch);

/**
 * Given a character from a world file, returns the corresponding diagonalDirectionT
 * for that character.  Throws an invalid_argument error if the parameter does not
 * correspond to a diagonal direction.
 */
diagonalDirectionT CharToDiagonal(char ch);

#endif