/**
 * Feel free to use this grid implementation in your assignment - just make a note
 * of it if you do.  For those of you who did not complete the grid assignment,
 * please remember the Honor Code and don't submit this file for Assignment 1!
 */

#ifndef grid_h_included__
#define grid_h_included__

#include <algorithm>
#include <utility>
using namespace std;

template<typename ElemType> class grid 
{  
public:   
	grid(); 
	grid(int width, int height);  
	grid(const grid& other);   
	grid& operator= (const grid& other); 
	~grid();  
        
	void clear();   
	void resize(int width, int height); 
  
	int getWidth() const { return width; } 
	int getHeight() const { return height; }  
	bool empty() const { return width == 0 || height == 0; }   
	int size() const { return width * height; } 
  
	ElemType& getAt(int x, int y);    
	const ElemType& getAt(int x, int y) const; 
   
	typedef ElemType* iterator;  
	typedef const ElemType* const_iterator;     
  
	iterator begin() { return array; }  
	iterator end()   { return array + width * height; }  
	const_iterator begin() const { return array; } 
	const_iterator end() const { return array + width * height; } 
  
	iterator getIteratorToElement(int x, int y) { return &getAt(x, y); }  
	const_iterator getIteratorToElement(int x, int y) const { return &getAt(x, y); } 
  
	pair<int, int> getCoordinatesForIterator(const_iterator itr) const; 
  
	class MutableReference 
	{  
	public: 
		friend class grid;  
		ElemType& operator[](int y) {return ref->getAt(x, y);} 
	private:  
		MutableReference(grid* source, int x) : ref(source), x(x) {} 
		grid * ref;  
		int x;  
	}; 
  
	class ImmutableReference 
	{  
	public: 
		friend class grid;  
		const ElemType& operator[](int y) const { return ref->getAt(x, y); } 
	private:  
		ImmutableReference(const grid* source, int x) : ref(source), x(x) {} 
		const grid * ref;  
		int x;  
	};  
 
	friend class MutableReference;  
	friend class ImmutableReference;  
	MutableReference   operator[](int x); 
	ImmutableReference operator[](int x) const; 
 
	bool operator <  (const grid& other) const;  
	bool operator <= (const grid& other) const; 
	bool operator == (const grid& other) const;  
	bool operator != (const grid& other) const; 
	bool operator >= (const grid& other) const;  
	bool operator >  (const grid& other) const; 
private:  
	ElemType* array; 
	int width;  
	int height;  
  
	void deleteData(); 
	void copyOther(const grid& other);  
}; 
  
template<typename T> 
typename grid<T>::MutableReference grid<T>::operator[](int x)  
{ 
	return MutableReference(this, x);  
} 
  
template<typename T> 
typename grid<T>::ImmutableReference grid<T>::operator[](int x) const  
{ 
	return ImmutableReference(this, x);  
} 
  
template<typename T> 
grid<T>::grid() : array(NULL), width(0), height(0)  
{ 
}  
 
template<typename T>  
grid<T>::grid(int width, int height) : array(new T[width * height]), width(width), height(height) 
{  
} 
  
template<typename T> 
grid<T>::~grid()  
{ 
	deleteData();  
} 
  
template<typename T> 
grid<T>::grid(const grid& other)  
{ 
	copyOther(other);  
} 
  
template<typename T> 
void grid<T>::deleteData()  
{ 
	delete [] array;  
	width = height = 0;  
	array = NULL;  
} 
  
template<typename T> 
void grid<T>::copyOther(const grid& other)  
{ 
	array = new T[other.size()];  
	width = other.width;  
	height = other.height; 
	copy(other.begin(), other.end(), begin());  
} 
  
template<typename T>  
grid<T>& grid<T>::operator= (const grid& other) 
{  
	if(this != &other) 
	{  
		deleteData(); 
		copyOther(other);  
	} 
	return *this;  
} 
  
template<typename T> 
void grid<T>::clear()  
{ 
	deleteData();  
} 
  
template<typename T>  
void grid<T>::resize(int width, int height)  
{ 
	deleteData();  
	array = new T[width * height]; 
	this->width = width;  
	this->height = height; 
}  
 
template<typename T> 
T& grid<T>::getAt(int x, int y)  
{ 
	/* Nifty trick to implement this function in terms of the const overload.  The
	 * first cast converts this object to const so we can call the const version,
	 * and the second strips the const-ness off the result.
	 */
	return const_cast<T&>(static_cast<const grid*>(this)->getAt(x, y));  
} 
  
template<typename T> 
const T& grid<T>::getAt(int x, int y) const  
{ 
	return array[x + y * width];  
} 
 
template<typename T>  
pair<int, int> grid<T>::getCoordinatesForIterator(typename grid<T>::const_iterator itr) const 
{  
	/* ptrdiff_t is a type exported by <cstdlib> that represents the difference of two
	 * pointers.
	 */
	ptrdiff_t dist = itr - begin(); 
	return make_pair(dist % width, dist / width);  
}  
 
template<typename T>  
bool grid<T>::operator <(const grid& other) const 
{  
	/* Lexicographically compare by dimensions. */  
	if(width < other.width) 
		return true;  
	if(width > other.width) 
		return false;  
	if(height < other.height) 
		return true;  
	if(height > other.height) 
		return false;  
 
	/* Perform a lexicographical comparison. */
	return lexicographical_compare(array, array + size(), other.array, other.array + other.size());
} 
  
template<typename T> 
bool grid<T>::operator >=(const grid& other) const  
{ 
	return !(*this < other);  
} 
  
template<typename T> 
bool grid<T>::operator ==(const grid& other) const  
{  
	return !(*this < other) && !(other < *this); 
} 
  
template<typename T> 
bool grid<T>::operator !=(const grid& other) const  
{  
	return (*this < other) || (other < *this); 
}  
 
template<typename T> 
bool grid<T>::operator >(const grid& other) const  
{ 
	return other < *this;  
} 
  
template<typename T> 
bool grid<T>::operator <= (const grid& other) const  
{  
	return !(other < *this); 
}  
  
#endif 