/*************************************
 * Main.cpp
 *
 * Main entry point into the MyPhoton game.
 * Feel free to modify this file and its contents!
 */

#include <iostream>
#include <fstream>
#include <string>
#include <stdexcept>
#include "photon-graphics.h"
#include "photon-controller.h"
#include "tile.h"
#include "direction.h"
#include "grid.h"
#include "genlib.h" // Appease the compiler.  Do not use CS106-specific features!
using namespace std;

/**
 * A helper struct that represents a laserlet.  Feel free to modify or
 * disregard this struct as you see fit.
 */
struct laserT
{
	directionT dir;
	int x, y;
};

/**
 * Helper function for LoadWorldSkeleton that figures out which tile is being
 * read at position (x, y) in the world.  You will probably need to modify this
 * function to get tile information.
 */
void ReadSingleTile(ifstream& input, int x, int y)
{
	/* Read in the next level component. */
	char identifier;
	input >> identifier;

	/* Based on what we've read, react acoordingly. */
	switch(identifier)
	{
	case 'W': // Wall tile
		// TODO: Create a Wall tile at (x, y)
		break;
	case '.': // Floor tile
		// TODO: Create an Empty tile at (x, y)
		break;
	default:
		{
			/* All other types of tiles have extra information associated with them
			 * that we're going to need to read in.  So, factor that code out to here.
			 */
			char ch;
			input >> ch;
		
			switch(identifier)
			{
			case 'M': // Mirror
			{
				const diagonalDirectionT dir = CharToDiagonal(ch);
				// TODO: Create a mirror at (x, y) facing in direction dir
				break;
			}
			case 'B': // Beam-splitter
			{
				const diagonalDirectionT dir = CharToDiagonal(ch);
				// TODO: Create a beam-splitter at (x, y) facing in direction dir
				break;
			}
			case 'L': // Laser
			{
				const directionT dir = CharToDirection(ch);
				// TODO: Create a laser at (x, y) facing in direction dir
				break;
			}
			case 'S': // Sensor
			{
				const directionT dir = CharToDirection(ch);
				// TODO: Create a sensor at (x, y) facing in direction dir
				break;
			}
			case 'T': // T-Mirror
			{
				const directionT dir = CharToDirection(ch);
				// TODO: Create a T-mirror at (x, y) facing in direction dir
				break;
			}
			default:
				throw runtime_error("Unexpected character: " + string(1, identifier));
			}
		}
	}
}

/**
 * A skeletal implementation of a world-loading function.  This function will
 * attempt to load in a world file and will do all of the proper error handling,
 * throwing runtime_error or invalid_argument exceptions as necessary.
 * However, the function does not actually create any data and you will need to
 * fill in the appropriate sections to have the parser work correctly.
 *
 * @param filename The name of the file to open.
 */
void LoadWorldSkeleton(const string& filename)
{
	/* The IOStream library exports a function called 'exceptions' that lets you
	 * configure whether or not the IO operations should throw exceptions on
	 * an error.  This is the behavior we want here, so we enable it by default.
	 * From now on, any read error will trigger an exception of type ios_base::failure.
	 */
	ifstream input;
	input.exceptions(ios::badbit | ios::failbit);

	try
	{
		input.open(filename.c_str());
		
		/* Read in the width and height of the world.  If they're out of bounds,
		 * report an error.
		 */
		int width, height;
		input >> width >> height;
		if(width <= 0 || height <= 0 || width >= PhotonGraphics::MAX_DIMENSIONS ||
		   height >= PhotonGraphics::MAX_DIMENSIONS)
		   throw runtime_error("World dimensions out of bounds.");

		/* Iterate over the grid reading in data.  Note that the files are stored
		 * upside-down, where the top line of the file has the highest index in the
		 * grid.  This is because the graphics also use an inverted coordinate system,
		 * so by storing it this way it's easier to edit the level files.
		 */
		for(int y = height - 1; y >= 0; --y)
			for(int x = 0; x < width; ++x)
				ReadSingleTile(input, x, y);
	}
	catch(const ios_base::failure&)
	{
		/* Convert IO errors into runtime_errors. */
		throw runtime_error("Error reading file.");
	}
}

/* Straight out of lecture code. */
string GetLine()
{
	string result;
	getline(cin, result);
	return result;
}

/**
 * Main program loop is currently a skeleton.  All it does it sit in a loop
 * trying to read files.
 */
int main()
{
	while(true)
	{
		try
		{
			/* Read in filename, quit on empty file. */
			cout << "Enter filename (RETURN to quit): ";
			string input = GetLine();
			if(input.empty()) break;

			LoadWorldSkeleton(input);
		}
		/* exception is the exception class at the root of the exception
		 * hierarchy defined in <stdexcept>.  invalid_argument, runtime_error,
		 * etc. are all subtypes of exception.  Consequently, catching objects
		 * of type exception will catch any exception thrown by <stdexcept>.
		 */
		catch(const exception & ex)
		{
			cout << ex.what() << endl;
		}
	}
	return 0;
}