/**
 * An interface that is used by PhotonGraphics to communicate events back to the main program.
 * You will need to make a derived class that inherits from PhotonController in order to interface
 * with the PhotonGraphics object.  DO NOT MODIFY THIS FILE.
 */

#ifndef PhotonController_included
#define PhotonController_included

class PhotonController
{
public:
	/**
	 * The PhotonGraphics object will call this function when the game is about to start.
	 * You should perform any needed initialization here.
	 */
	virtual void onGameStart() = 0;

	/**
	 * The PhotonGraphics object will call this function when the game has ended.
	 * You should perform any needed cleanup code here.
	 */
	virtual void onGameEnd() = 0;

	/**
	 * This function will be called when the PhotonGraphics object needs to redraw the board.
	 * Although it could be optimized for efficiency, this function should redraw the entire
	 * board.
	 */
	virtual void drawBoard() = 0;

	/**
	 * This function is called when the user clicks on a tile to try to select it.  The
	 * x and y parameters represent the x and y (in board coordinates) of where the click
	 * occurred and are guaranteed to be within the range specified to PhotonGraphics in
	 * the resizeWorld function.  The final parameter indicates whether or not this is the
	 * first of the two tiles being selected.  This function should return true if the selection
	 * is valid and false otherwise.
	 *
	 * @param x The x coordinate of the tile being considered.
	 * @param y The y coordinate of the tile being considered.
	 * @param firstTile Whether this is the first tile being considered.
	 * @return Whether the selection is valid.
	 */
	virtual bool trySelectTile(int x, int y, bool firstTile) = 0;

	/**
	 * This function is invoked when the user has selected two tiles for which trySelectTile
	 * has returned true.  swapTiles should move the piece at position (x1, y1) to position
	 * (x2, y2).  The values (x1, y1) and (x2, y2) are guaranteed to have been the values
	 * approved of by trySelectTile.
	 *
	 * @param x1 The x coordinate of the first tile to swap.
	 * @param y1 The y coordiante of the first tile to swap.
	 * @param x2 The x coordinate of the second tile to swap.
	 * @param y2 The y coordiante of the second tile to swap.
	 */
	virtual void swapTiles(int x1, int y1, int x2, int y2) = 0;

	/**
	 * This function is invoked at several different points to determine whether or not the
	 * player has solved the puzzle.  It should return true if the puzzle is solved, false
	 * otherwise.
	 *
	 * @return Whether the player has solved the puzzle.
	 */
	virtual bool isPuzzleSolved() = 0;

	/**
	 * Virtual destructor allows us to destroy PhotonController objects
	 * and have cleanup code in derived classes execute.
	 */
	virtual ~PhotonController() {}
};

#endif