/**
 * Exports the type tileT, which is used by PhotonGraphics to display the different tile
 * pictures.  You will need to use the functions provided by this interface when drawing
 * the board.  The tileT type is provided as an enum, but its actual contents are hidden.
 * To get the appropriate tileT for a given tile, use the functions exported here.  DO NOT
 * MODIFY THIS FILE.
 */

#ifndef Tile_included
#define Tile_included

#include "direction.h"

/* Definition of tileT.  You can use these constants if you choose, but it's
 * strongly recommended that you use the helper functions provided below instead.
 */
enum tileT {
	tileBlock = 8, tileWall, tileEmpty,
	tileMirrorDownRight, tileMirrorUpRight,
	tileTMirrorNorth, tileTMirrorSouth, tileTMirrorEast, tileTMirrorWest,
	tileBeamSplitterDownRight, tileBeamSplitterUpRight,
	tileOffSensorNorth, tileOffSensorSouth, tileOffSensorEast, tileOffSensorWest,
	tileOnSensorNorth, tileOnSensorSouth, tileOnSensorEast, tileOnSensorWest,
	tileLaserNorth, tileLaserSouth, tileLaserEast, tileLaserWest,
	InvalidTile
};

/**
 * Returns the picture of a laser pointing in the direction specified by dir.
 * Throws a runtime_error exception if the direction is invalid.
 */
tileT GetLaserPicture(directionT dir);

/**
 * Returns the picture of a sensor pointing in the direction specified by dir.
 * The isOn parameter specifies whether the returned tile is in the "on" state.
 * Throws a runtime_error exception if the direction is invalid.
 */
tileT GetSensorPicture(directionT dir, bool isOn);

/**
 * Returns the picture of a mirror oriented in the direction specified by dir.
 * Throws a runtime_error exception if the direction is invalid.
 */
tileT GetMirrorPicture(diagonalDirectionT dir);

/**
 * Returns the picture of a T-mirror whose point is facing in the direction specified by
 * dir.  Throws a runtime_error exception if the direction is invalid.
 */
tileT GetTMirrorPicture(directionT dir);

/**
 * Returns the picture of a beam-splitter oriented in the direction specified by dir.
 * Throws a runtime_error exception if the direction is invalid.
 */
tileT GetBeamSplitterPicture(diagonalDirectionT dir);

/**
 * Returns the picture of the empty tile.
 */
tileT GetEmptyTilePicture();

/**
 * Returns the picture of the wall tile.
 */
tileT GetWallTilePicture();

#endif